import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

//NumberRange.java

/**
 * I model a simple animation drawing
 * 
 * I contain a collection of "Drawable" types that
 * I ask to animate (move) at intervals.
 * 
 * Also, a "Drawable" type knows how to draw itself.
 *
 */
public class AnimationDrawing {

	public enum Actions {
		NO_ACTION, ADD_CIRCLE, ANIMATE, CLEAR
	};

	//----------------------------------------------------------------
	//Ivars

	private List<Animatable> animationObjects;
	private Frame boundary; //drawing boundary

	//----------------------------------------------------------------
	//Constructors

	public AnimationDrawing(Frame aBoundary) {
		super();
		this.initialize();
		this.boundary = aBoundary;
	}

	protected void initialize() {
		this.animationObjects = new ArrayList<>();
	}

	//----------------------------------------------------------------
	//Instance Methods

		public void doActionWith(Actions action, Object actionArgument) {		
		switch (action) {
			case ADD_CIRCLE:
				this.addCircle((Integer)actionArgument);
				break;		
			case ANIMATE:
				this.animate();
				break;				
			case CLEAR:
				this.clear();
				break;
			default:
				break;
		}
	}

	private void clear() {
		this.animationObjects.clear();
	}
	
	private void addCircle(int countToAdd) {
		//Add circle animation object at c random position, radius, and direction
		Random rand = new Random();
		for (int i = 0; i < countToAdd; i++) {
			Circle circ = new Circle(this.boundary.randomPointInside(), rand.nextInt(5, 15));
			circ.randomizeDisplayProperties();
			AnimationObject animObject = new AnimationObject(circ);
			this.add(animObject);
		}
	}	
	
	/** Performs the next animation step */
	public void animate() {
		for (Animatable nextObject: this.animationObjects)
			nextObject.forwardIn(this.boundary);
	}

	/** Draw the entire drawing on graphics "g" */
	public void drawOn(Graphics g) {
		for (Animatable d : this.animationObjects) {
			d.drawOn(g);
		}
	}

	//----------------------------------------------------------------
	//Helper Methods	

	public void add(AnimationObject drawable) {
		this.animationObjects.add(drawable);
	}

}
