import java.awt.Graphics;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

public class AnimationDrawingController extends Controller {

	//---------------------------------------------------
	//Ivars	

	private AnimationDrawing model;
	private View view;
	private AnimationDrawing.Actions action;
	private Object actionArgument;
	private Timer timer;

	//---------------------------------------------------
	//Constructors	

	public AnimationDrawingController(View aView) {
		super();
		this.view = aView;
		this.resetModel();
		this.clearAction();
	}

	//---------------------------------------------------
	//Getters

	public AnimationDrawing getModel() {
		return model;
	}

	//---------------------------------------------------
	//Overrides (not needed) -- TODO -- make overrides optional	

	@Override
	public void refresh() {
	}

	@Override
	public void apply() {
	}

	//---------------------------------------------------
	//Hooking Up Events	

	protected void hookUpEvents() {
		((JButton) this.get("animateButton")).addActionListener(evt -> this.clickedGo());
		((JButton) this.get("stopButton")).addActionListener(evt -> this.clickedStop());
		((JButton) this.get("clearButton")).addActionListener(evt -> this.clickedClear());
		((JButton) this.get("addCirclesButton")).addActionListener(evt -> this.clickedAddCircle());
	}

	//---------------------------------------------------
	//Framework Methods

	@Override
	public View getView() {
		return this.view;
	}

	//---------------------------------------------------
	//Instance Methods

	private void clearAction() {
		this.action = AnimationDrawing.Actions.NO_ACTION;
		this.actionArgument = null;
	}

	JLabel getMessageLabel() {
		return (JLabel) this.get("message");
	}

	//---------------------------------------------------
	//Action Handlers

	private void clickedGo() {
		//this.resetModel();		
		this.startTimer();
	}

	private void clickedStop() {
		if (this.timer != null)
			this.timer.cancel();
		this.timer = null;
		//this.resetModel();
	}

	private void clickedAddCircle() {
		Integer count = this.promptForInteger("Enter number of circles\nto randomly add\n(maximum 100)",
				"Please enter valid integer");
		if (count != null)
			if (count <= 100) {
				this.actionArgument = count;
				this.runAction(AnimationDrawing.Actions.ADD_CIRCLE);
			} else {
				this.showError("Input Error", "Please enter integer of 100 or less");

	}

	}

	private void clickedClear() {
		this.runAction(AnimationDrawing.Actions.CLEAR);
	}

	private void runAction(AnimationDrawing.Actions anAction) {
		AnimationDrawingView v = (AnimationDrawingView) this.view;
		this.action = anAction;
		try {
			v.setActionRunning(true);
			v.getDrawingArea().repaint();
		} finally {
		}
	}

	public void drawOn(Graphics g) {
		this.model.doActionWith(this.action, this.actionArgument);
		this.model.drawOn(g);
		this.clearAction();
	}

	//---------------------------------------------------
	//Animation Handlers	

	/** Handle the next clock tick */
	public void tick() {
		this.runAction(AnimationDrawing.Actions.ANIMATE);
	}

	public void startTimer() {
		this.timer = new Timer();
		//TODO -- put period (third param - 10) into variable
		//(timer task, initial delay , rate [ms])
		timer.schedule(new DrawingTask(this), 0, 10);
	}

	//---------------------------------------------------
	//Helper Methods	

	public void resetModel() {
		Frame boundary = ((AnimationDrawingView) this.view).getBoundary();
		this.model = new AnimationDrawing(boundary);
	}

	private String promptForString(String prompt) {
		Object[] choices = null; //simple input
		Icon icon = null;
		String s = (String) JOptionPane.showInputDialog(this.getViewMainComponent(), prompt, "",
				JOptionPane.PLAIN_MESSAGE, icon, choices, "1");
		if (s == null || s.length() == 0)
			return null;
		return s;
	}

	private Integer promptForInteger(String prompt, String errorMessage) {
		String str = this.promptForString(prompt);
		if (str == null)
			return null;
		Integer number = null;
		try {
			number = Integer.valueOf(str);
		} catch (Exception e) {
			showError("Input Error", errorMessage);
		}
		return number;
	}

	private void showError(String title, String errorMessage) {
		JOptionPane.showMessageDialog(this.getViewMainComponent(), errorMessage, title, JOptionPane.ERROR_MESSAGE);
	}

}


//======================================================================
//Helper Classes

class DrawingTask extends TimerTask {
	private AnimationDrawingController controller;

	public DrawingTask(AnimationDrawingController aController) {
		this.controller = aController;
	}

	public void run() {
		controller.tick();
	}
}
