import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

@SuppressWarnings("serial")

public class AnimationDrawingView extends View {
	
	//---------------------------------------------------
	//Ivars	
	
	private AnimationDrawingController controller;
	private JPanel drawingArea;
	private boolean actionRunning;
	
	public AnimationDrawingController getController() {
		return controller;
	}	
	
	public JPanel getDrawingArea() {
		return drawingArea;
	}
	
	public boolean isActionRunning() {
		return actionRunning;
	}

	public void setActionRunning(boolean running) {
		this.actionRunning = running;
	}	
	
	//---------------------------------------------------
	//Constructors	

	AnimationDrawingView() {
		//The next line calls the one-arg constructor
		//this("MVC Example");
		super("SmartNumber View");
		this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);		
		this.actionRunning = false;
		
	    JPanel outer = createVertGroup();
	    this.drawingArea = this.createGraphicsPanel();
		outer.add(this.drawingArea);
		
		controller = new AnimationDrawingController(this);		
		
		//Action Buttons
		JPanel buttonBar = this.createButtonGroup(this.controller);
		outer.add(buttonBar);		
		
		//Message Panel
		outer.add(this.createMessagePanel());		
				
		this.add(outer);
		this.setBounds(800, 100, 1000, 1000);
		this.setVisible(true);
		
		this.controller.hookUpEvents();
		this.controller.refresh();		
	}
	
	//---------------------------------------------------
	//Instance Methods
	
	public Frame getBoundary() {
		return new Frame(0, 0, this.drawingArea.getWidth(), this.drawingArea.getHeight());
	}
	
	//---------------------------------------------------
	//Building View Groups
	
	private JPanel createGraphicsPanel() {
		JPanel panel = new DrawingArea(this);
		panel.setName("graphicsPane");
		panel.setSize(1000, 800);
		return panel;
	}
	
	private JPanel createButtonGroup(
			@SuppressWarnings("unused") AnimationDrawingController controller) {
		JPanel buttons = createHorizGroup();
		JButton button;

		button = new JButton("Add Circles");
		button.setName("addCirclesButton");
		buttons.add(button);
		
		button = new JButton("Animate");
		button.setName("animateButton");
		buttons.add(button);
		
		button = new JButton("Stop");
		button.setName("stopButton");
		buttons.add(button);		
		
		button = new JButton("Clear");
		button.setName("clearButton");
		buttons.add(button);
		
		return buttons;
	}	
	
	private Component createMessagePanel() {
		JPanel messagePanel = createHorizGroup();
		JLabel message = new JLabel(" ");
		message.setName("message");
		message.setForeground(Color.red);
		message.setFont(new Font("Serif", Font.BOLD, 24));
		messagePanel.add(message);
		return messagePanel;
	}	
	
	//---------------------------------------------------
	//Main (Program Launch Point / Entry Point)	
	
	public static void main(String args[]) {
		new AnimationDrawingView();
	}
}

class DrawingArea extends JPanel {

	private static final long serialVersionUID = 1L;
	
	private AnimationDrawingView view;
	
	public DrawingArea(AnimationDrawingView parent) {
		super();
		this.view = parent;
	}

	@Override 
    public void paintComponent(Graphics g) {
         super.paintComponent(g);
         if (this.view.isActionRunning())
        	 this.view.getController().drawOn(g);
         this.view.setActionRunning(false);
    }
}

