import java.awt.Graphics;
import java.util.Random;

/** models a two-dimensional point (x, y) */
public class AnimationObject implements Animatable {
	
	//----------------------------------------------------------------
	//Ivars
	
	private Drawable drawable;		//We hold a drawable object
	private double direction;		//Direction in radians (modeling unit) - east is 0 (counterclockwise)
	private int distance;			//Distance of each animation step
	private Vector velocityVector;	//Combination of direction and distance form the velocityVector
									//The velocityVector dictates the movement for each animation step
	
	//----------------------------------------------------------------
	//Accessors (Getters & Getters)
	
	//----------------------------------------------------------------
	//Constructors
	
	public AnimationObject(Drawable aDrawable) {
		this(aDrawable, (new Random()).nextInt(0, 360), (new Random()).nextInt(1, 3));
	}	
	
	public AnimationObject(Drawable aDrawable, int aDirectionAsDegrees, int aDistance) {
		super();
		this.drawable = aDrawable;
		this.distance = aDistance;		
		this.direction(aDirectionAsDegrees);
	}	
	
	//----------------------------------------------------------------
	//Overrides for Animatable	
	
	@Override
	public void forwardIn(Frame f) {
		Drawable drawable = this.drawable;
		drawable.moveBy(this.velocityVector.multiply(this.distance));
		if (!drawable.isInside(f))
			drawable.moveTo(f.middle());
	}

	/** Set the direction of this object (degrees) */
	@Override
	public void direction(int aDirection) {
		//For user convenience, we receive direction in degrees
		//However, for modeling, we want radians, so we convert
		this.direction = this.degreesToRadians(aDirection);
		this.updateVelocityVector();
	}
	
	@Override
	public void drawOn(Graphics g) {
		this.drawable.drawOn(g);
	}	

	//----------------------------------------------------------------
	//Helper Methods
	
	@Override
	public String toString() {
		return "AnimationObject [drawable=" + drawable + ", direction=" + direction + "]";
	}

	private double degreesToRadians(int aDirection) {
		//180 degrees per PI radians
		//So, PI/180 radians/degree
		return aDirection * (Math.PI / 180.0); 
	}
	
	private void updateVelocityVector() {
		//For example, say our direction is 0.6435 radians (36.87 degrees)
		//This relates to a vector (4, 3) -- e.g. a (3 4 5) right triangle
		//Because the hypotenuse of a unit vector is 1, then the
		//unit vector becomes (cos(direction), sin(direction))
		//y-axis downward
		Vector directionUnitVector = 
			new Vector(Math.cos(direction), -Math.sin(direction));
		this.velocityVector = directionUnitVector.multiply(this.distance);
	}
	
	//----------------------------------------------------------------
	//Instance Methods	

}
