import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Graphics2D;

/** models a circle in 2D space, with center and radius */
public class Circle extends GraphicsElement implements Drawable {
	
	//----------------------------------------------------------------
	//Ivars
	
	//Geometric properties
	private Point center;
	private int radius;
	
	//----------------------------------------------------------------
	//Accessors (Getters & Setters)
	
	//----------------------------------------------------------------
	//Constructors
	
	/** Construct circle from point (x1, y1) and radius */
	public Circle(int x1, int y1, int r) {
		this(new Point(x1, y1), r);
	}
	
	/** Construct circle from point and radius */	
	public Circle(Point aCenter, int r) {
		super();
		this.center = aCenter;
		this.radius = r;
	}	

	//----------------------------------------------------------------
	//Instance Methods
	
	@Override
	public void moveBy(Vector vector) {
		this.center.moveBy(vector);
	}	
	
	//----------------------------------------------------------------
	//Helpers	

	private int getDiameter() {
		return 2 * this.radius;
	}	
		
	//----------------------------------------------------------------
	//Drawable type overrides
	
	@Override
	public String toString() {
		return "Circle [center=" + center + ", radius=" + radius + "]";
	}
	
	@Override
	public void drawOn(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;		
		if (this.getColor() != null)
			g2.setColor(this.getColor());
		if (this.getThickness() > 0)
			g2.setStroke(new BasicStroke(this.getThickness()));
		Point leftTop = this.center.minus(this.radius);
		int d = this.getDiameter();
		g.drawOval(leftTop.getX(), leftTop.getY(), d, d);
	}

	/** Returns true if this circle is inside frame "f" */
	@Override
	public boolean isInside(Frame f) {
		//Simple implemention here considering only our center
		return f.contains(this.center);
	}
	
	/** Moves this object to point "position" */
	@Override
	public void moveTo(Point position) {
		this.center = position;
	}
	
}
