import java.util.Random;

/*
Class: Frame
For: Chapter "Chapter 602 - Logic" -- Challenge #2  
11/28/2020
lt left top
rt right top
rb right bottom
lb left bottom

NOTE WELL -- In computer graphics y axis is 
oriented downward (top to bottom). 
X axis is left to right.
*/

public class Frame {

	private int leftX, topY, rightX, bottomY;

	/** Construct Frame from left, top, right bottom (see example above) */
	public Frame(int newLeftX, int newTopY, int newRightX, int newBottomY) {
		this.leftX = newLeftX;
		this.topY = newTopY;
		this.rightX = newRightX;
		this.bottomY = newBottomY;
	}

	@Override
	public String toString() {
		String delim = ", ";
		return "(leftX, topY, rightX, bottomY): (" + getLeftX() + delim + getTopY() + delim + getRightX() + delim
				+ getBottomY() + ")";
	}

	/** Return left top point */
	public Point lt() {
		return new Point(getLeftX(), getTopY());
	}

	/** Return right top point */
	public Point rt() {
		return new Point(getRightX(), getTopY());
	}

	/** Return right bottom point */
	public Point rb() {
		return new Point(getRightX(), getBottomY());
	}

	/** Return left bottom point */
	public Point lb() {
		return new Point(getLeftX(), getBottomY());
	}

	/** Return left edge (x value) of frame (see example above) */
	public int getLeftX() {
		return this.leftX;
	}

	/** Return top edge (y value) of frame (see example above) */
	public int getTopY() {
		return this.topY;
	}

	/** Return right edge (y value) of frame (see example above) */
	public int getRightX() {
		return this.rightX;
	}

	/** Return bottom edge (y value) of frame (see example above) */
	public int getBottomY() {
		return this.bottomY;
	}

	/** Return true if this frame intersects other */
	public boolean intersects(Frame other) {
		return contains(other.lt()) || contains(other.rt()) || contains(other.rb()) || contains(other.lb());
	}

	/** Return true if this frame contains other (i.e. is "other" inside this frame) */
	public boolean contains(Frame other) {
		return this.containsHoriz(other) && this.containsVert(other);
	}

	/** Return true if we contain "other" frame horizontally (x values) */
	public boolean containsHoriz(Frame other) {
		return this.containsX(other.getLeftX()) && this.containsX(other.getRightX());
	}

	/** Return true if we contain "other" frame vertically (y values) */
	public boolean containsVert(Frame other) {
		return this.containsY(other.getTopY()) && this.containsY(other.getBottomY());
	}

	/** Return true if we contain point "p" */
	public boolean contains(Point p) {
		return containsX(p.getX()) && containsY(p.getY());
	}

	/** Return true if we contain the x (if x is between our left and right) */
	public boolean containsX(int x) {
		return x >= getLeftX() && x <= getRightX();
	}

	/** Return true if we contain the y (if y is between our top and bottom) */
	public boolean containsY(int y) {
		return y >= getTopY() && y <= getBottomY();
	}

	/** Shrink every side by parameter "shrink" */
	public void shrinkByAmount(int shrink) {
		this.leftX += shrink;
		this.topY += shrink;
		this.rightX -= shrink;
		this.bottomY -= shrink;
	}

	/** Return width of this frame */
	public int getWidth() {
		return this.rightX - this.leftX;
	}

	/** Return height of this frame */
	public int getHeight() {
		return this.bottomY - this.topY;
	}

	/** Return a point, that is the middle of this frame */
	public Point middle() {
		int x, y;
		x = this.leftX + (int) Math.round(this.getWidth() / 2.0);
		y = this.topY + (int) Math.round(this.getHeight() / 2.0);
		return new Point(x, y);
	}
	
	/** Returns a random point that is inside this frame */
	public Point randomPointInside() {
		int x, y;
		Random rand = new Random();
		x = rand.nextInt(this.getLeftX(), this.getRightX());
		y = rand.nextInt(this.getTopY(), this.getBottomY());
		return new Point(x, y);				
	}

}