import java.awt.Component;

import javax.swing.JTextArea;
import javax.swing.JTextField;

public abstract class Controller {
	
	//---------------------------------------------------
	//Constructors	

	public Controller() {
	}
	
	//---------------------------------------------------
	//Abstract Methods (must be implemented by subclass)
	
	/** refresh the view from the model */
	public abstract void refresh();
	
	/** apply the view values to the model */
	public abstract void apply();
	
	/** apply the view values to the model */
	public abstract View getView();
	
	//---------------------------------------------------
	//Instance Methods
	
	public Component getViewMainComponent() {
		return this.getView().getMainComponent();
	}
	
	//---------------------------------------------------
	//Helpers

	/** Convenience method to get view elem -- e.g., button, text field, label, etc */
	public Component get(String elemName) {
		return View.paneNamed(this.getView(), elemName);
	}

	/** Returns the value component of a field (i.e., the component that holds the value) */
	public JTextField valueComponent(String fieldName) {
		return (JTextField)View.paneNamed(View.paneNamed(this.getView(), fieldName), "input");
	}	
	
	public void setFieldValue(String fieldName, Object fieldText) {
		valueComponent(fieldName).setText(fieldText.toString());
	}	
	
	public void setMultiLineFieldValue(String fieldName, Object fieldText) {
		JTextArea textArea = (JTextArea)View.paneNamed(View.paneNamed(this.getView(), fieldName), "input");
		textArea.setText(fieldText.toString());		
	}		
	
	public String getFieldValue(String fieldName) {
		return valueComponent(fieldName).getText();
	}	
	
	public Integer getFieldValueAsInteger(String fieldName) {
		return Integer.valueOf(getFieldValue(fieldName));
	}	
	
	public Double getFieldValueAsDouble(String fieldName) {
		return Double.valueOf(getFieldValue(fieldName));
	}	
	
	
	
}