import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

//NumberRange.java

/**
 * I model a drawing
 * 
 * I contain a collection of "Drawable" types.
 * 
 * A "Drawable" type cheifly knows how to draw itself.
 *
 */
public class Drawing {

	public enum Actions {
		NO_ACTION, ADD_LINES, ADD_POLYLINES, CLEAR
	};

	//----------------------------------------------------------------
	//Ivars

	private List<Drawable> drawables;
	private Frame boundary; //drawing boundary

	//----------------------------------------------------------------
	//Constructors

	public Drawing(Frame aBoundary) {
		super();
		this.initialize();
		this.boundary = aBoundary;
	}

	protected void initialize() {
		this.drawables = new ArrayList<>();
	}

	//----------------------------------------------------------------
	//Instance Methods

	public void doActionWith(Drawing.Actions action, Object actionArgument) {
		switch (action) {
			case ADD_LINES:
				this.addLine();
				break;
			case ADD_POLYLINES:
				this.addPolylines((Integer)actionArgument);
				break;				
			case CLEAR:
				this.clear();
				break;
			case NO_ACTION:
				break;
		}
	}

	private void clear() {
		this.drawables.clear();
	}

	/** Add a line to the drawing with random start and end points */
	public void addLine() {
		int x1, y1, x2, y2;
		Frame b = this.boundary;
		Random r = new Random();
		x1 = r.nextInt(b.getLeftX(), b.getRightX() + 1);
		y1 = r.nextInt(b.getTopY(), b.getBottomY() + 1);
		x2 = r.nextInt(b.getLeftX(), b.getRightX() + 1);
		y2 = r.nextInt(b.getTopY(), b.getBottomY() + 1);
		Line newLine = new Line(x1, y1, x2, y2);
		newLine.setRandomColor();
		this.add(newLine);
	}
	
	public void addPolylines(int count) {
		Polyline polyline = new Polyline();
		polyline.setRandomColor();
		polyline.setThickness(5);		
		Frame bndry = this.boundary;
		List<Point> randomPoints = new ArrayList<>();
		for (int i = 0; i < count + 1; i++)
			randomPoints.add(bndry.randomPointInside());
		polyline.addFromPoints(randomPoints);
		this.add(polyline);
	}

	/** Draw the entire drawing on graphics "g" */
	public void drawOn(Graphics g) {
		for (Drawable d : this.drawables) {
			d.drawOn(g);
		}
	}

	//----------------------------------------------------------------
	//Helper Methods	

	public void add(Drawable drawable) {
		this.drawables.add(drawable);
	}
}
