import java.awt.Graphics;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JOptionPane;

public class DrawingController extends Controller {

	//---------------------------------------------------
	//Ivars	

	private Drawing model;
	private View view;
	private Drawing.Actions action;
	private Object actionArgument;

	//---------------------------------------------------
	//Constructors	

	public DrawingController(Drawing aModel, View aView) {
		super();
		this.model = aModel;
		this.view = aView;
		this.clearAction();
	}

	//---------------------------------------------------
	//Getters

	public Drawing getModel() {
		return model;
	}

	//---------------------------------------------------
	//Overrides (not needed) -- TODO -- make overrides optional	

	@Override
	public void refresh() {
	}

	@Override
	public void apply() {
	}

	//---------------------------------------------------
	//Hooking Up Events	

	protected void hookUpEvents() {
		((JButton) this.get("addLinesButton")).addActionListener(evt -> this.clickedAddLines());
		((JButton) this.get("addPolylinesButton")).addActionListener(evt -> this.clickedAddPolylinesButton());
		((JButton) this.get("clearButton")).addActionListener(evt -> this.clickedClear());
	}

	//---------------------------------------------------
	//Framework Methods

	@Override
	public View getView() {
		return this.view;
	}

	//---------------------------------------------------
	//Instance Methods

	private void clearAction() {
		this.action = Drawing.Actions.NO_ACTION;
		this.actionArgument = null;
	}

	//---------------------------------------------------
	//Action Handlers

	private void clickedAddLines() {
		this.doAction(Drawing.Actions.ADD_LINES);
	}

	private void clickedAddPolylinesButton() {
		Integer count = this.promptForInteger("Enter size (number of points)", "Please enter valid integer");
		if (count != null) {
			this.actionArgument = count;
			this.doAction(Drawing.Actions.ADD_POLYLINES);
		}
	}

	private void clickedClear() {
		this.doAction(Drawing.Actions.CLEAR);
	}

	private void doAction(Drawing.Actions anAction) {
		DrawingView v = (DrawingView) this.view;
		this.action = anAction;
		try {
			v.setActionRunning(true);
			v.getDrawingArea().repaint();
		} finally {
		}
	}

	public void drawOn(Graphics g) {
		this.model.doActionWith(this.action, this.actionArgument);
		this.model.drawOn(g);
		this.clearAction();
	}

	//---------------------------------------------------
	//Helper Methods

	private String promptForString(String prompt) {
		Object[] choices = null; //simple input
		Icon icon = null;
		String s = (String) JOptionPane.showInputDialog(this.getViewMainComponent(), prompt, "",
				JOptionPane.PLAIN_MESSAGE, icon, choices, "1");
		if (s == null || s.length() == 0)
			return null;
		return s;
	}

	private Integer promptForInteger(String prompt, String errorMessage) {
		String str = this.promptForString(prompt);
		if (str == null)
			return null;
		Integer number = null;
		try {
			number = Integer.valueOf(str);
		} catch (Exception e) {
			showError("Input Error", errorMessage);
		}
		return number;
	}

	private void showError(String title, String errorMessage) {
		JOptionPane.showMessageDialog(this.getViewMainComponent(), errorMessage, title, JOptionPane.ERROR_MESSAGE);
	}
}
