import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;

/** 
 * A polyline models a "many-line" meaning many lines joined together,
 * It's a helpful in modeling real world things (roads, boundary edges,
 * etc). Curves can be approximated with shorter lines.
 *  */
public class Polyline extends GraphicsElement implements Drawable {
	
	//----------------------------------------------------------------
	//Ivars
	
	private List<Line> lines;		//Our "many-line" (list of lines)
	
	//----------------------------------------------------------------
	//Accessors (Getters & Getters)
	
	//----------------------------------------------------------------
	//Constructors
	
	/** Construct empty polyline */
	public Polyline() {
		this(new ArrayList<>());	//empty polyline
	}

	/** Construct polyline containing param linesList */	
	public Polyline(List<Line> linesList) {
		super();
		this.lines = linesList;
		for (Line nextLine: this.lines)
			nextLine.setThickness(5);
	}
	
	/** Returns Polyline using ordered points (used to construct many-lines) -- static factory constructor */	
	public static Polyline fromPoints(List<Point> points) {
		//Construct many lines using ordered collection of points
		Polyline polyline = new Polyline();
		polyline.addFromPoints(points);
		return polyline;
	}	

	//----------------------------------------------------------------
	//Instance Methods

	/** Draw line to "end" point on graphics "g" */
	public void drawOn(Graphics g) {
		for (Line nextLine: this.lines)
			nextLine.drawOn(g);
	}
	
	public void add(Line line) {
		if (line.getColor() == null)
			line.setColor(this.getColor());
		if (line.getThickness() == 0)
			line.setThickness(this.getThickness());
		this.lines.add(line);
	}
	
	/** Add lines from ordered points */	
	public void addFromPoints(List<Point> points) {
		if (points.size() < 2) return;
		for (int i = 1; i < points.size(); i++)
			this.add(new Line(points.get(i - 1), points.get(i)));
	}	
	
}
