import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

public abstract class View extends JFrame {

	private static final long serialVersionUID = 1L;

	View(String label) {
		super(label);
	}
	
	//---------------------------------------------------
	//Building View Groups	
	
	/** create vertical group */
	protected static JPanel createVertGroup() {
	    final JPanel panel = new JPanel();
	    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
	    return panel;
	}	
	
	/** create horizontal group */
	protected static JPanel createHorizGroup() {
	    final JPanel panel = new JPanel();
	    panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
	    return panel;
	}	

	//---------------------------------------------------
	//Building View Elements

	/** create field "width: __________ */
	protected static JPanel createField(@SuppressWarnings("unused") View view,
										String nm, String label, boolean editable) {
		final JPanel field = new JPanel();
		field.setName(nm);
		field.setAlignmentX(Component.LEFT_ALIGNMENT);
		field.setLayout(new BoxLayout(field, BoxLayout.X_AXIS));
		field.add(createLabel(label, JLabel.CENTER, new Dimension(175, 30)));

		JTextField input = new JTextField("", 40);
		input.setEditable(editable);
		input.setName("input");

		Dimension size = new Dimension(150, 30);
		input.setPreferredSize(size);
		input.setMaximumSize(size);
		input.setMinimumSize(size);
		
		input.addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {
				if (input.isEditable())
					input.selectAll();
			}
			public void focusLost(FocusEvent e) {}
		});

		field.add(input);

		return field;
	}

	/** create multiline field */
	protected static JPanel createMultiLineField(View view, String nm,
												String label, boolean editable, 
												Dimension inputSz) {
		final JPanel field = new JPanel();
		field.setName(nm);
		field.setAlignmentX(Component.LEFT_ALIGNMENT);
		field.setLayout(new BoxLayout(field, BoxLayout.X_AXIS));
		
		field.add(createLabel(label, JLabel.TOP, new Dimension(175, (int)Math.round(inputSz.getHeight()))));

		JTextArea input = new JTextArea();
		input.setEditable(editable);
		input.setBackground(view.getBackground());
		input.setName("input");
		input.setBorder(BorderFactory.createLineBorder(new Color(162, 208, 225)));

		input.setPreferredSize(inputSz);
		input.setMaximumSize(inputSz);
		input.setMinimumSize(inputSz);

		field.add(input);
		return field;
	}

	/** create multiline field */
	protected static JLabel createLabel(String label, int verticalAlignment,
										Dimension labelSz) {
		JLabel jlabel = new JLabel(label + ": ", SwingConstants.RIGHT);
		jlabel.setVerticalAlignment(verticalAlignment);
		jlabel.setPreferredSize(labelSz);
		jlabel.setMaximumSize(labelSz);
		jlabel.setMinimumSize(labelSz);
		return jlabel;
	}
	
	//---------------------------------------------------	
	//Instance Methods
	
	public Component getMainComponent() {
		return this.getContentPane();
	}
	
	//---------------------------------------------------
	//Static	

	public static Component paneNamed(Component root, String paneName) {
		if (root == null)
			return null;
		if (root.getName() != null && root.getName().equals(paneName))
			return root;
		if (root instanceof Container)
			for (Component eachChild : ((Container) root).getComponents()) {
				Component match = paneNamed(eachChild, paneName);
				if (match != null)
					return match;
			}
		return null;
	}

}