import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

//NumberRange.java

/**
 * I model a simple animation drawing
 * 
 * I contain a collection of "Drawable" types that
 * I ask to animate (move) at intervals.
 * 
 * Also, a "Drawable" type knows how to draw itself.
 *
 */
public class AnimationDrawing {

	public enum Actions {
		NO_ACTION, ADD_CIRCLE, ANIMATE, CLEAR
	};

	//----------------------------------------------------------------
	//Ivars

	private List<AnimationObject> animationObjects;
	private Frame boundary; //drawing boundary

	//----------------------------------------------------------------
	//Constructors

	public AnimationDrawing(Frame aBoundary) {
		super();
		this.initialize();
		this.boundary = aBoundary;
		//Add 10 pixels fuzz to prevent objects from passing edges
		this.boundary.shrinkByAmount(10);
	}

	protected void initialize() {
		this.animationObjects = new ArrayList<>();
	}

	//----------------------------------------------------------------
	//Instance Methods

	public void doAction(AnimationDrawing.Actions action) {
		switch (action) {
			case ADD_CIRCLE:
				this.addCircle();
				break;		
			case ANIMATE:
				this.animate();
				break;				
			case CLEAR:
				this.clear();
				break;
			default:
				break;
		}
	}

	private void clear() {
		this.animationObjects.clear();
	}
	
	private void addCircle() {
		//Add circle animation object at c random position, radius, and direction
		Random rand = new Random();
		Circle circ = new Circle(this.boundary.randomPointInside(), rand.nextInt(5, 15));
		circ.randomizeDisplayProperties();
		AnimationObject animObject = new AnimationObject(circ);
		this.add(animObject);
	}	
	
	/** Performs the next animation step */
	public void animate() {
		for (AnimationObject nextObject: this.animationObjects)
			nextObject.forwardIn(this.boundary);
	}

	/** Draw the entire drawing on graphics "g" */
	public void drawOn(Graphics g) {
		for (AnimationObject d : this.animationObjects) {
			d.drawOn(g);
		}
	}

	//----------------------------------------------------------------
	//Helper Methods	

	public void add(AnimationObject drawable) {
		this.animationObjects.add(drawable);
	}

}
