import java.awt.Graphics;
import java.util.Random;

/** models a two-dimensional point (x, y) */
public class AnimationObject implements Animatable {
	
	//----------------------------------------------------------------
	//Ivars
	
	private Drawable drawable;		//We hold a drawable object
	private Vector velocityVector;	//Combination of direction and distance form the velocityVector
									//The velocityVector dictates the movement for each animation step
	
	//----------------------------------------------------------------
	//Accessors (Getters & Getters)
	
	//----------------------------------------------------------------
	//Constructors
	
	public AnimationObject(Drawable aDrawable) {
		this(aDrawable, (new Random()).nextInt(0, 360), (new Random()).nextInt(1, 3));
	}	
	
	public AnimationObject(Drawable aDrawable, int aDirectionAsDegrees, int aDistance) {
		super();
		this.drawable = aDrawable;
		this.direction(aDirectionAsDegrees, aDistance);
	}	
	
	//----------------------------------------------------------------
	//Overrides for Animatable	
	
	@Override
	public void forwardIn(Frame f) {
		Drawable drawable = this.drawable;
		drawable.moveBy(this.velocityVector);
		if (!drawable.isInside(f)) {
			this.reflectOffOf(f);
			drawable.moveBy(this.velocityVector);
		}
	}

	/** Set the direction of this object (degrees) */
	public void direction(int aDirection, int distance) {
		//For user convenience, we receive direction in degrees
		//However, for modeling, we want radians, so we convert
		this.updateVelocityVector(aDirection, distance);
	}
	
	@Override
	public void drawOn(Graphics g) {
		this.drawable.drawOn(g);
	}	

	//----------------------------------------------------------------
	//Helper Methods
	
	@Override
	public String toString() {
		return "AnimationObject [velocityVector=" + velocityVector + "]";
	}

	private void updateVelocityVector(int direction, int length) {
		this.velocityVector = Vector.fromDirectionLength(direction, length); 
	}
	
	/** Reflects this object off an edge in "f" (if this object is outside an edge) */
	private void reflectOffOf(Frame f) {
		Frame.RelativePositon pos = this.drawable.positionRelativeTo(f);
		Vector vv = this.velocityVector;
		Vector vvNew = null;
		if (pos == Frame.RelativePositon.INSIDE)
			return;
		if (pos == Frame.RelativePositon.LEFT || pos == Frame.RelativePositon.RIGHT) 
			vvNew = new Vector(-vv.getX(), vv.getY());
		else if  (pos == Frame.RelativePositon.ABOVE || pos == Frame.RelativePositon.BELOW) 
			vvNew = new Vector(vv.getX(), -vv.getY());
		this.velocityVector = vvNew;
	}

	//----------------------------------------------------------------
	//Instance Methods	

}
