import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.Random;

/** models a circle in 2D space, with center and radius */
public class Circle implements Drawable {
	
	//----------------------------------------------------------------
	//Ivars
	
	//Geometric properties
	private Point center;
	private int radius;
	
	//Cosmetic/display properties
	private Color color;
	//Line thickness (roughly equivalent to 1/72 of an inch)	
	private int thickness;
	
	//----------------------------------------------------------------
	//Accessors (Getters & Setters)
	
	public void setColor(Color color) {
		this.color = color;
	}
	
	public void setThickness(int thickness) {
		this.thickness = thickness;
	}	
	
	//----------------------------------------------------------------
	//Constructors
	
	/** Construct circle from point (x1, y1) and radius */
	public Circle(int x1, int y1, int r) {
		this(new Point(x1, y1), r);
	}
	
	/** Construct circle from point and radius */	
	public Circle(Point aCenter, int r) {
		super();
		this.center = aCenter;
		this.radius = r;
	}	

	//----------------------------------------------------------------
	//Instance Methods
	
	public void randomizeDisplayProperties() {
		this.setRandomThickness();
		this.setRandomColor();
	}
	
	public void setRandomThickness() {
		this.thickness = (new Random()).nextInt(1, 50);
	}	
	
	public void setRandomColor() {
		this.color = GraphicsToolbox.randomColor();
	}
	
	@Override
	public void moveBy(Vector vector) {
		this.center.moveBy(vector);
	}	
	
	//----------------------------------------------------------------
	//Helpers	

	private int getDiameter() {
		return 2 * this.radius;
	}	
		
	//----------------------------------------------------------------
	//Drawable type overrides
	
	@Override
	public String toString() {
		return "Circle [center=" + center + ", radius=" + radius + "]";
	}
	
	@Override
	public void drawOn(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;		
		if (this.color != null)
			g2.setColor(this.color);
		if (this.thickness > 0)
			g2.setStroke(new BasicStroke(this.thickness));
		Point leftTop = this.center.minus(this.radius);
		int d = this.getDiameter();
		g.drawOval(leftTop.getX(), leftTop.getY(), d, d);
	}

	/** Returns true if this circle is inside frame "f" */
	@Override
	public boolean isInside(Frame f) {
		//Simple implemention here considering only our center
		return f.contains(this.boundary());
	}
	
	/** Moves this object to point "position" */
	@Override
	public void moveTo(Point position) {
		this.center = position;
	}

	@Override
	public Frame.RelativePositon positionRelativeTo(Frame f) {
		return f.relativePositionOf(this.boundary());
	}

	@Override
	public Frame boundary() {
		Point leftTop = this.center.minus(this.radius);
		int d = this.getDiameter();
		int leftX, topY;
		leftX = leftTop.getX();
		topY = leftTop.getY();
		return new Frame(leftX, topY, leftX + d, topY + d);
	}
	
}
