
/** Vector
 * Models a direction (x, y) in 2D space.
*/
public class Vector {
	
	//----------------------------------------------------------------
	//Ivars
	
	private double x;
	private double y;

	//----------------------------------------------------------------
	//Accessors (Getters & Getters)
	
	public double getX() {
		return x;
	}
	
	public double getY() {
		return y;
	}
	
	public void setX(double x) {
		this.x = x;
	}	
	
	public void setY(double y) {
		this.y = y;
	}

	//----------------------------------------------------------------
	//Constructors
	
	public static Vector fromDirectionLength(int direction, int length) {
		//For example, say our direction is 0.6435 radians (36.87 degrees)
		//This relates to a vector (4, 3) -- e.g. a (3 4 5) right triangle
		//Because the hypotenuse of a unit vector is 1, then the
		//unit vector becomes (cos(direction), sin(direction))
		//y-axis downward
		Vector newVector = 
			new Vector(Math.cos(direction), -Math.sin(direction));
		newVector.length(length);
		return newVector;
		//return newVector.multiply(length);
	}	
	
	public Vector() {
		this(0, 0);	//default point is origin (0, 0)
	}

	public Vector(double x, double y) {
		super();
		this.x = x;
		this.y = y;
	}
	
	//----------------------------------------------------------------
	//Instance Methods

	@Override
	public String toString() {
		return "Vector [x=" + x + ", y=" + y + "]";
	}
	
	/** Returns this vector as an angle (radians) */
	public double angle() {
		if (this.x == 0) return 0.0;
		return Math.asin(this.y / this.x);
	}
	
	public void angle(double newAngle) {
		double len = this.length();
		this.setX(Math.cos(newAngle));
		this.setY(Math.sin(newAngle));
		this.length(len);
	}	

	/** Returns new vector with x and y multiplied by paran "distance" */	
	public Vector multiply(double distance) {
		return new Vector(this.x * distance, this.y * distance);
	}	
	
	/** Returns new vector with x and y rounded to nearest integers */
	public Vector rounded() {
		return new Vector(Math.round(this.x), Math.round(this.y));
	}
	
	/** Returns the length of this vector */
	public double length() {
		return Math.sqrt(this.x*this.x + this.y*this.y);
	}
	
	/** Sets the length of this vector */
	public void length(double newLength) {
		Vector uv = this.unitVector();
		Vector newVector = uv.multiply(newLength);
		this.setX(newVector.getX());
		this.setY(newVector.getY());
	}	
	
	/** Returns a unit vector for this vector */
	public Vector unitVector() {
		double len = this.length();
		return new Vector(this.x/len, this.y/len);
	}

	//----------------------------------------------------------------
	//Main
	
	public static void main(String[] args) {
		Vector v = new Vector(4, 3);
		System.out.println(v.unitVector());
	}
	
}
