import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

//NumberRange.java

/**
 * I model a drawing
 * 
 * I contain a collection of "Drawable" types.
 * 
 * A "Drawable" type cheifly knows how to draw itself.
 *
 */
public class Drawing {

	public enum Action {
		NO_ACTION, ADD_LINE, ADD_POLYLINE, CLEAR
	};

	//----------------------------------------------------------------
	//Ivars

	private List<Drawable> drawables;
	private Frame boundary; //drawing boundary

	//----------------------------------------------------------------
	//Constructors

	public Drawing(Frame aBoundary) {
		super();
		this.initialize();
		this.boundary = aBoundary;
	}

	protected void initialize() {
		this.drawables = new ArrayList<>();
	}

	//----------------------------------------------------------------
	//Instance Methods

	public void doAction(Drawing.Action action) {
		switch (action) {
		case ADD_LINE:
			this.addLine();
			break;
		case CLEAR:
			this.clear();
			break;
		default:
			break;
		}
	}

	private void clear() {
		System.out.println("CLEAR");
		this.drawables.clear();
	}

	/** Add a line to the drawing with random start and end points */
	public void addLine() {
		int x1, y1, x2, y2;
		Frame b = this.boundary;
		Random r = new Random();
		x1 = r.nextInt(b.getLeftX(), b.getRightX() + 1);
		y1 = r.nextInt(b.getTopY(), b.getBottomY() + 1);
		x2 = r.nextInt(b.getLeftX(), b.getRightX() + 1);
		y2 = r.nextInt(b.getTopY(), b.getBottomY() + 1);
		Line newLine = new Line(x1, y1, x2, y2);
		newLine.setRandomThickness();
		newLine.setRandomColor();
		this.add(newLine);
	}

	/** Draw the entire drawing on graphics "g" */
	public void drawOn(Graphics g) {
		for (Drawable d : this.drawables) {
			d.drawOn(g);
		}
	}

	public void addPolyline(List<Point> userPoints) {
		Polyline newPolyline = Polyline.fromPoints(userPoints);
		newPolyline.setColor(GraphicsToolbox.randomColor());
		this.add(newPolyline);
	}

	public void addLine(List<Point> userPoints) {
		Line newLine = Line.fromPoints(userPoints);
		newLine.setRandomColor();
		newLine.setRandomThickness();
		this.add(newLine);
	}

	public void addDrawable(Action action, List<Point> points) {
		switch (action) {
		case ADD_POLYLINE:
			this.addPolyline(points);
			break;
		case ADD_LINE:
			this.addLine(points);
			break;
		default:
			break;
		}
	}

	public int expectedPointCountFor(Drawing.Action action) {
		switch (action) {
		case ADD_POLYLINE:
			return Integer.MAX_VALUE; //no limit
		case ADD_LINE:
			return 2;
		default:
			return -1;
		}
	}

	//----------------------------------------------------------------
	//Helper Methods	

	public void add(Drawable drawable) {
		this.drawables.add(drawable);
	}

}
