import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.List;
import java.util.Random;

/** models a two-dimensional point (x, y) */
public class Line implements Drawable {
	
	//----------------------------------------------------------------
	//Ivars
	
	//Geometric properties
	private Point start;
	private Point end;
	
	//Cosmetic/display properties
	private Color color;
	//Line thickness (roughly equivalent to 1/72 of an inch)	
	private int thickness;
	
	//----------------------------------------------------------------
	//Accessors (Getters & Setters)
	
	public void setColor(Color color) {
		this.color = color;
	}
	
	public void setThickness(int thickness) {
		this.thickness = thickness;
	}	
	
	//----------------------------------------------------------------
	//Constructors
	
	/** Construct line from points -- param must be size 2 */	
	public static Line fromPoints(List<Point> points) {
		if (points.size() != 2)
			throw new RuntimeException("Expecting two points when constructing line 'fromPoints'"); 
		return new Line(points.get(0), points.get(1));
	}	
	
	/** Construct line from point (x1, y1) to point (x2, y2) */
	public Line(int x1, int y1, int x2, int y2) {
		this(new Point(x1, y1), new Point(x2, y2));
	}
	
	/** Construct line from point p1 to point p2 */	
	public Line(Point p1, Point p2) {
		super();
		this.start = p1;
		this.end = p2;
	}	

	//----------------------------------------------------------------
	//Instance Methods	
	
	public void setRandomThickness() {
		this.thickness = (new Random()).nextInt(1, 50);
	}
	
	public void setRandomColor() {
		this.color = GraphicsToolbox.randomColor();
	}			
	
	//----------------------------------------------------------------
	//Drawable type overrides
	
	@Override
	public String toString() {
		return "Line [start=" + start + ", end=" + end + "]";
	}

	@Override
	public void drawOn(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;		
		if (this.color != null)
			g2.setColor(this.color);
		g2.setStroke(new BasicStroke(this.thickness));
		this.start.drawToOn(this.end, g2);
	}

}
